package main

import (
	"cicv-data-closedloop/aarch64/pjisuv/common/config"
	commonService "cicv-data-closedloop/aarch64/pjisuv/common/service"
	"cicv-data-closedloop/common/config/c_log"
	"cicv-data-closedloop/common/util"
	"net/rpc"
	"os"
	"runtime"
	"time"
)

func init() {
	runtime.GOMAXPROCS(1)
	// 初始化日志配置
	c_log.InitLog("/mnt/media/sda1/cicv-data-closedloop/log/", "pjisuv-control")
	// 初始化本地配置文件（第1处配置，在本地文件）
	config.InitLocalConfig()
	// 初始化Oss连接信息
	config.InitOssConfig()
	// 初始化业务逻辑配置信息，配置文件在oss上（第2处配置，在oss文件）
	config.InitCloudConfig()
}

func main() {
	lastStatus := "NONE"
	//  轮询任务接口判断是否有更新
	for {
		time.Sleep(time.Duration(2) * time.Second)
		// 1 获取当前设备的任务的 status
		status, err := config.GetStatus(config.PlatformConfig.TaskConfigId)
		if err != nil {
			c_log.GlobalLogger.Error("获取配置status失败：", err)
			continue
		}
		// 2 判断 status
		// UN_CHANGE 没有新的任务，无需更改
		// CHANGE 有新的任务，需要杀死旧的任务并重启
		// NONE 设备没有配置任务，需要杀死旧的任务
		if status == "UN_CHANGE" {
			lastStatus = "UN_CHANGE"
			continue
		} else if status == "CHANGE" || status == "NONE" {
			if lastStatus == "NONE" && status == "NONE" {
				continue
			}
			// 3 发送rpc信号杀死两个服务，并重启程序
			if lastStatus == "NONE" && status == "CHANGE" {
				if _, err := util.ExecuteWithPath(config.LocalConfig.RestartCmd.Dir, config.LocalConfig.RestartCmd.Name, config.LocalConfig.RestartCmd.Args...); err != nil {
					c_log.GlobalLogger.Info("启动新程序失败,【path】=", config.LocalConfig.RestartCmd.Dir, "【cmd】=", config.LocalConfig.RestartCmd.Name, config.LocalConfig.RestartCmd.Args, "：", err)
					os.Exit(-1)
				}
				c_log.GlobalLogger.Info("启动任务，本地执行启动命令：【path】=", config.LocalConfig.RestartCmd.Dir, "【cmd】=", config.LocalConfig.RestartCmd.Name, config.LocalConfig.RestartCmd.Args)
				lastStatus = status
				c_log.GlobalLogger.Info("获取数据闭环平台最新配置。")
				config.InitPlatformConfig()
				continue
			}
			var killArgs *commonService.KillSignal
			if lastStatus == "UN_CHANGE" && status == "CHANGE" {
				killArgs = &commonService.KillSignal{DropUploadData: config.PlatformConfig.DropUploadData, Restart: true}
				c_log.GlobalLogger.Info("更新任务，发送rpc重启信号到本地：", killArgs)
			}
			if lastStatus == "UN_CHANGE" && status == "NONE" {
				killArgs = &commonService.KillSignal{DropUploadData: config.PlatformConfig.DropUploadData, Restart: false}
				c_log.GlobalLogger.Info("杀死任务，发送rpc结束信号到本地：", killArgs)
			}

			KillRpcClient, err := rpc.Dial("tcp", config.LocalConfig.Node.Ip+":"+config.CloudConfig.RpcPort)
			if err != nil {
				c_log.GlobalLogger.Error("创建rpc客户端连接master失败：", err)
				// 此处关闭client会报错
				continue
			}

			reply := 0
			if err = KillRpcClient.Call("KillService.Kill", killArgs, &reply); err != nil {
				c_log.GlobalLogger.Error("发送rpc请求到master失败：", err)
				//TODO 这里可能会报错 unexpected EOF 但是不影响，先注释 close 和 continue
				//KillRpcClient.Close()
				//continue
			}
			lastStatus = status
			c_log.GlobalLogger.Info("获取数据闭环平台最新配置。")
			config.InitPlatformConfig()
			KillRpcClient.Close()
		} else {
			c_log.GlobalLogger.Error("未知的采集任务状态。【status】=", status)
		}
	}
}
